/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

var _require = require('../../../util/numbers'),
    int16Table = _require.int16Table;

/**
 * Canonical truth table for the `Add16` gate.
 */


var TRUTH_TABLE = int16Table([{ a: 0, b: 0, out: 0 }, { a: 0, b: 65535, out: 65535 }, { a: 65535, b: 65535, out: 65534 }, { a: 43690, b: 21845, out: 65535 }, { a: 15555, b: 4080, out: 19635 }, { a: 4660, b: 39030, out: 43690 }]);

/**
 * A 16-bit integer adder.
 */

var Add16 = function (_BuiltInGate) {
  _inherits(Add16, _BuiltInGate);

  function Add16() {
    _classCallCheck(this, Add16);

    return _possibleConstructorReturn(this, (Add16.__proto__ || Object.getPrototypeOf(Add16)).apply(this, arguments));
  }

  _createClass(Add16, [{
    key: 'eval',

    /**
     * IN a[16], b[16];
     * OUT out[16];
     *
     * Abstract:
     *
     *   HalfAdder(a=a[0], b=b[0], sum=out[0], carry=c1);
     *   FullAdder(a=a[1], b=b[1], c=c1, sum=out[1], carry=c2);
     *   ...
     *
     * Technically use JS + operator on 16-bit values.
     */
    value: function _eval() {
      var a = this.getInputPins()[0].getValue();
      var b = this.getInputPins()[1].getValue();

      this.getOutputPins()[0].setValue(a + b);
    }
  }]);

  return Add16;
}(BuiltInGate);

/**
 * Specification of the `Add16` gate.
 */


Add16.Spec = {
  name: 'Add16',

  description: 'A 16-bit integer adder: out = int16(a + b)',

  inputPins: [{ name: 'a', size: 16 }, { name: 'b', size: 16 }],

  outputPins: [{ name: 'out', size: 16 }],

  truthTable: TRUTH_TABLE
};

module.exports = Add16;