/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

var _require = require('../../../util/numbers'),
    int16Table = _require.int16Table;

/**
 * Canonical truth table for the `And16` gate.
 */


var TRUTH_TABLE = int16Table([{ a: 0, b: 0, out: 0 }, { a: 0, b: 65535, out: 0 }, { a: 65535, b: 65535, out: 65535 }, { a: 43690, b: 21845, out: 0 }, { a: 15555, b: 4080, out: 3264 }, { a: 4660, b: 39030, out: 4148 }]);

/**
 * A bitwise 16-bit And gate.
 */

var And16 = function (_BuiltInGate) {
  _inherits(And16, _BuiltInGate);

  function And16() {
    _classCallCheck(this, And16);

    return _possibleConstructorReturn(this, (And16.__proto__ || Object.getPrototypeOf(And16)).apply(this, arguments));
  }

  _createClass(And16, [{
    key: 'eval',

    /**
     * IN a[16], b[16];
     * OUT out[16];
     *
     * for i = 0..15: out[i] = (a[i] & b[i])
     *
     * Abstract:
     *
     *   And(a=a[0], b=b[0], out=out[0]);
     *   And(a=a[1], b=b[1], out=out[1]);
     *   ...
     *
     * Technically use JS bitwise operations at needed index.
     */
    value: function _eval() {
      var a = this.getInputPins()[0].getValue();
      var b = this.getInputPins()[1].getValue();

      // In JS implementation doesn't differ from the simple `And` gate.
      this.getOutputPins()[0].setValue(a & b);
    }
  }]);

  return And16;
}(BuiltInGate);

/**
 * Specification of the `And16` gate.
 */


And16.Spec = {
  name: 'And16',

  description: 'Implements bitwise 16-bit And & operation.',

  inputPins: [{ name: 'a', size: 16 }, { name: 'b', size: 16 }],

  outputPins: [{ name: 'out', size: 16 }],

  truthTable: TRUTH_TABLE
};

module.exports = And16;