/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

/**
 * Canonical truth table for the `DMux` gate.
 */
var TRUTH_TABLE = [{ in: 0, sel: 0, out1: 0, out2: 0 }, { in: 0, sel: 1, out1: 0, out2: 0 }, { in: 1, sel: 0, out1: 1, out2: 0 }, { in: 1, sel: 1, out1: 0, out2: 1 }];

/**
 * 1 bit demultiplexer.
 * if sel=0 {out1=in; out2=0} else {out1=0; out2=in}
 */

var DMux = function (_BuiltInGate) {
  _inherits(DMux, _BuiltInGate);

  function DMux() {
    _classCallCheck(this, DMux);

    return _possibleConstructorReturn(this, (DMux.__proto__ || Object.getPrototypeOf(DMux)).apply(this, arguments));
  }

  _createClass(DMux, [{
    key: 'eval',
    value: function _eval() {
      var _in = this.getInputPins()[0].getValue();
      var sel = this.getInputPins()[1].getValue();

      this.getOutputPins()[0].setValue(sel === 0 ? _in : 0);
      this.getOutputPins()[1].setValue(sel === 0 ? 0 : _in);
    }
  }]);

  return DMux;
}(BuiltInGate);

/**
 * Specification of the `DMux` gate.
 */


DMux.Spec = {
  name: 'DMux',

  description: ['Implements 1-bit demultiplexer (DMux) gate.', '', '{out1 = in, out2 = 0 }, when sel = 0', '{out1 = 0,  out2 = in}, when sel = 1'].join('\n'),

  inputPins: ['in', 'sel'],
  outputPins: ['out1', 'out2'],

  truthTable: TRUTH_TABLE
};

module.exports = DMux;