/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

/**
 * Canonical truth table for the `FullAdder` gate.
 */
var TRUTH_TABLE = [{ a: 0, b: 0, c: 0, sum: 0, carry: 0 }, { a: 0, b: 0, c: 1, sum: 1, carry: 0 }, { a: 0, b: 1, c: 0, sum: 1, carry: 0 }, { a: 0, b: 1, c: 1, sum: 0, carry: 1 }, { a: 1, b: 0, c: 0, sum: 1, carry: 0 }, { a: 1, b: 0, c: 1, sum: 0, carry: 1 }, { a: 1, b: 1, c: 0, sum: 0, carry: 1 }, { a: 1, b: 1, c: 1, sum: 1, carry: 1 }];

/**
 * A FullAdder.
 *
 * The `sum` returns the LSB of the sum of the three bits a, b and c.
 * The `carry` returns the carry bit.
 */

var FullAdder = function (_BuiltInGate) {
  _inherits(FullAdder, _BuiltInGate);

  function FullAdder() {
    _classCallCheck(this, FullAdder);

    return _possibleConstructorReturn(this, (FullAdder.__proto__ || Object.getPrototypeOf(FullAdder)).apply(this, arguments));
  }

  _createClass(FullAdder, [{
    key: 'eval',

    /**
     * t = a + b + c
     * sum = t % 2
     * carry = t / 2
     */
    value: function _eval() {
      var a = this.getInputPins()[0].getValue();
      var b = this.getInputPins()[1].getValue();
      var c = this.getInputPins()[2].getValue();

      var t = a + b + c;

      this.getOutputPins()[0].setValue(t % 2); // sum
      this.getOutputPins()[1].setValue(Math.trunc(t / 2)); // carry
    }
  }]);

  return FullAdder;
}(BuiltInGate);

/**
 * Specification of the `FullAdder` gate.
 */


FullAdder.Spec = {
  name: 'FullAdder',

  description: ['Implements 3-bits adder (full-adder) gate.', '', 'The `sum` returns LSB (the least significant bit) of the sum', 'of the three bits `a`, `b` and `c`.', '', 'The `carry` returns the carry bit.'].join('\n'),

  inputPins: ['a', 'b', 'c'],
  outputPins: ['sum', 'carry'],

  truthTable: TRUTH_TABLE
};

module.exports = FullAdder;