/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

/**
 * Canonical truth table for the `HalfAdder` gate.
 */
var TRUTH_TABLE = [{ a: 0, b: 0, sum: 0, carry: 0 }, { a: 0, b: 1, sum: 1, carry: 0 }, { a: 1, b: 0, sum: 1, carry: 0 }, { a: 1, b: 1, sum: 0, carry: 1 }];

/**
 * A HalfAdder.
 * `sum` returns the LSB of the sum of the two bits a and b.
 * `carry` returns the carry bit.
 */

var HalfAdder = function (_BuiltInGate) {
  _inherits(HalfAdder, _BuiltInGate);

  function HalfAdder() {
    _classCallCheck(this, HalfAdder);

    return _possibleConstructorReturn(this, (HalfAdder.__proto__ || Object.getPrototypeOf(HalfAdder)).apply(this, arguments));
  }

  _createClass(HalfAdder, [{
    key: 'eval',

    /**
     * sum = a ^ b
     * carry = a & b
     */
    value: function _eval() {
      var a = this.getInputPins()[0].getValue();
      var b = this.getInputPins()[1].getValue();

      this.getOutputPins()[0].setValue(a ^ b);
      this.getOutputPins()[1].setValue(a & b);
    }
  }]);

  return HalfAdder;
}(BuiltInGate);

/**
 * Specification of the `HalfAdder` gate.
 */


HalfAdder.Spec = {
  name: 'HalfAdder',

  description: ['Implements 2-bits adder (half-adder) gate.', '', 'The `sum` returns LSB (the least significant bit) of the sum', 'of the two bits `a`, and `b`.', '', 'The `carry` returns the carry bit.'].join('\n'),

  inputPins: ['a', 'b'],
  outputPins: ['sum', 'carry'],

  truthTable: TRUTH_TABLE
};

module.exports = HalfAdder;