/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

var _require = require('../../../util/numbers'),
    int16Table = _require.int16Table;

/**
 * Canonical truth table for the `Inc16` gate.
 */


var TRUTH_TABLE = int16Table([{ in: 0, out: 1 }, { in: 65535, out: 0 }, { in: 5, out: 6 }, { in: 65531, out: 65532 }]);

/**
 * Adds the constant 1 to the input.
 */

var Inc16 = function (_BuiltInGate) {
  _inherits(Inc16, _BuiltInGate);

  function Inc16() {
    _classCallCheck(this, Inc16);

    return _possibleConstructorReturn(this, (Inc16.__proto__ || Object.getPrototypeOf(Inc16)).apply(this, arguments));
  }

  _createClass(Inc16, [{
    key: 'eval',

    /**
     * IN in[16];
     * OUT out[16];
     *
     * Abstract:
     *
     *   HalfAdder(a=in[0], b=1, sum=out[0], carry=c1);
     *   HalfAdder(a=in[1], b=c1, sum=out[1], carry=c2);
     *   ...
     *
     * Technically use JS + operator on 16-bit values.
     */
    value: function _eval() {
      var _in = this.getInputPins()[0].getValue();

      this.getOutputPins()[0].setValue(_in + 1);
    }
  }]);

  return Inc16;
}(BuiltInGate);

/**
 * Specification of the `Inc16` gate.
 */


Inc16.Spec = {
  name: 'Inc16',

  description: 'Adds the constant 1 to the input.',

  inputPins: [{ name: 'in', size: 16 }],

  outputPins: [{ name: 'out', size: 16 }],

  truthTable: TRUTH_TABLE
};

module.exports = Inc16;