/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

/**
 * Canonical truth table for the `Mux` gate.
 */
var TRUTH_TABLE = [{ a: 0, b: 0, sel: 0, out: 0 }, { a: 0, b: 0, sel: 1, out: 0 }, { a: 0, b: 1, sel: 0, out: 0 }, { a: 0, b: 1, sel: 1, out: 1 }, { a: 1, b: 0, sel: 0, out: 1 }, { a: 1, b: 0, sel: 1, out: 0 }, { a: 1, b: 1, sel: 0, out: 1 }, { a: 1, b: 1, sel: 1, out: 1 }];

/**
 * 1-bit 2-way multiplexor.
 * if sel=1 out=b else out=a.
 */

var Mux = function (_BuiltInGate) {
  _inherits(Mux, _BuiltInGate);

  function Mux() {
    _classCallCheck(this, Mux);

    return _possibleConstructorReturn(this, (Mux.__proto__ || Object.getPrototypeOf(Mux)).apply(this, arguments));
  }

  _createClass(Mux, [{
    key: 'eval',
    value: function _eval() {
      var a = this.getInputPins()[0].getValue();
      var b = this.getInputPins()[1].getValue();

      var sel = this.getInputPins()[2].getValue();

      this.getOutputPins()[0].setValue(sel === 0 ? a : b);
    }
  }]);

  return Mux;
}(BuiltInGate);

/**
 * Specification of the `Mux` gate.
 */


Mux.Spec = {
  name: 'Mux',

  description: ['Implements 1-bit 2-way multiplexor (Mux) gate.', '', 'out = a, when sel = 0', 'out = b, when sel = 1'].join('\n'),

  inputPins: ['a', 'b', 'sel'],
  outputPins: ['out'],

  truthTable: TRUTH_TABLE
};

module.exports = Mux;