/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

var _require = require('../../../util/numbers'),
    int16Table = _require.int16Table;

/**
 * Canonical truth table for the `Mux16` gate.
 */


var TRUTH_TABLE = int16Table([{ a: 0, b: 0, sel: 0, out: 0 }, { a: 0, b: 0, sel: 1, out: 0 }, { a: 0, b: 4660, sel: 0, out: 0 }, { a: 0, b: 4660, sel: 1, out: 4660 }, { a: 39030, b: 0, sel: 0, out: 39030 }, { a: 39030, b: 0, sel: 1, out: 0 }, { a: 43690, b: 21845, sel: 0, out: 43690 }, { a: 43690, b: 21845, sel: 1, out: 21845 }]);

/**
 * A 16-bit Mux gate.
 */

var Mux16 = function (_BuiltInGate) {
  _inherits(Mux16, _BuiltInGate);

  function Mux16() {
    _classCallCheck(this, Mux16);

    return _possibleConstructorReturn(this, (Mux16.__proto__ || Object.getPrototypeOf(Mux16)).apply(this, arguments));
  }

  _createClass(Mux16, [{
    key: 'eval',

    /**
     * IN a[16], b[16], sel;
     * OUT out[16];
     *
     * for i = 0..15 out[i] = a[i] if sel == 0
     *                        b[i] if sel == 1
     * Abstract:
     *
     *   Mux(a=a[0], b=b[0], sel=sel, out=out[0]);
     *   Mux(a=a[1], b=b[1], sel=sel, out=out[1]);
     *   ...
     *
     * Technically at JS implementation is the same as in 1-bit Mux.
     */
    value: function _eval() {
      var a = this.getInputPins()[0].getValue();
      var b = this.getInputPins()[1].getValue();

      var sel = this.getInputPins()[2].getValue();

      this.getOutputPins()[0].setValue(sel === 0 ? a : b);
    }
  }]);

  return Mux16;
}(BuiltInGate);

/**
 * Specification of the `Mux16` gate.
 */


Mux16.Spec = {
  name: 'Mux16',

  description: 'Implements bitwise 16-bit And & operation.',

  inputPins: [
  // Data pins.
  { name: 'a', size: 16 }, { name: 'b', size: 16 },

  // 1-bit selector.
  { name: 'sel', size: 1 }],

  outputPins: [{ name: 'out', size: 16 }],

  truthTable: TRUTH_TABLE
};

module.exports = Mux16;