/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

/**
 * Canonical truth table for the `Nand` gate.
 */
var TRUTH_TABLE = [{ a: 0, b: 0, out: 1 }, { a: 0, b: 1, out: 1 }, { a: 1, b: 0, out: 1 }, { a: 1, b: 1, out: 0 }];

/**
 * A bitwise 1-bit Nand (negative-And) gate.
 */

var Nand = function (_BuiltInGate) {
  _inherits(Nand, _BuiltInGate);

  function Nand() {
    _classCallCheck(this, Nand);

    return _possibleConstructorReturn(this, (Nand.__proto__ || Object.getPrototypeOf(Nand)).apply(this, arguments));
  }

  _createClass(Nand, [{
    key: 'eval',

    /**
     * Nand is the very basic chip on top of which any other chip can
     * be implemented: https://en.wikipedia.org/wiki/NAND_gate.
     *
     * It shares this property with the Nand gate.
     *
     * In the internal implementation we build it on top of the `&` operation.
     */
    value: function _eval() {
      var a = this.getInputPins()[0].getValue();
      var b = this.getInputPins()[1].getValue();

      this.getOutputPins()[0].setValue(0x1 - (a & b));
    }
  }]);

  return Nand;
}(BuiltInGate);

/**
 * Specification of the `Nand` gate.
 */


Nand.Spec = {
  name: 'Nand',

  description: ['Implements bitwise 1-bit Nand (negative-And) gate.', '', 'The "Nand" gate similarly to the "Nor" gate is a basic', 'building block for all other gates.'].join('\n'),

  inputPins: ['a', 'b'],
  outputPins: ['out'],

  truthTable: TRUTH_TABLE
};

module.exports = Nand;