/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

var _require = require('../../../util/numbers'),
    int16Table = _require.int16Table;

/**
 * Canonical truth table for the `Nor8Way` gate.
 */


var TRUTH_TABLE = int16Table([{ in: 0, out: 1 }, { in: 65535, out: 0 }, { in: 4112, out: 0 }, { in: 257, out: 0 }, { in: 9766, out: 0 }]);

/**
 * Or of 16 inputs into 1 output.
 * out=1 if all inputs are 0, and 0 otherwise.
 */

var Nor16Way = function (_BuiltInGate) {
  _inherits(Nor16Way, _BuiltInGate);

  function Nor16Way() {
    _classCallCheck(this, Nor16Way);

    return _possibleConstructorReturn(this, (Nor16Way.__proto__ || Object.getPrototypeOf(Nor16Way)).apply(this, arguments));
  }

  _createClass(Nor16Way, [{
    key: 'eval',

    /**
     * IN in[16];
     * OUT out;
     *
     * out = (in[0] nor in[1] nor ... nor in[7])
     *
     * Abstract:
     *
     *   Nor(a=in[0], b=in[1], out=out1);
     *   Nor(a=out1, b=in[2], out=out2);
     *   ...
     */
    value: function _eval() {
      var _in = this.getInputPins()[0].getValue();

      this.getOutputPins()[0].setValue(_in === 0 ? 1 : 0);
    }
  }]);

  return Nor16Way;
}(BuiltInGate);

/**
 * Specification of the `Nor16Way` gate.
 */


Nor16Way.Spec = {
  name: 'Nor16Way',

  description: 'Nor of 16 inputs into 1 output.',

  inputPins: [{ name: 'in', size: 16 }],

  outputPins: ['out'],

  truthTable: TRUTH_TABLE
};

module.exports = Nor16Way;