/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

var _require = require('../../../util/numbers'),
    int16Table = _require.int16Table;

/**
 * Canonical truth table for the `Not16` gate.
 */


var TRUTH_TABLE = int16Table([{ in: 0, out: 65535 }, { in: 65535, out: 0 }, { in: 43690, out: 21845 }, { in: 15555, out: 49980 }, { in: 4660, out: 60875 }]);

/**
 * A bitwise 16-bit Not gate.
 */

var Not16 = function (_BuiltInGate) {
  _inherits(Not16, _BuiltInGate);

  function Not16() {
    _classCallCheck(this, Not16);

    return _possibleConstructorReturn(this, (Not16.__proto__ || Object.getPrototypeOf(Not16)).apply(this, arguments));
  }

  _createClass(Not16, [{
    key: 'eval',

    /**
     * IN a[16];
     * OUT out[16];
     *
     * for i = 0..15: out[i] = ~a[i]
     *
     * Abstract:
     *
     *   Not(a=a[0], out=out[0]);
     *   Not(a=a[1], out=out[1]);
     *   ...
     *
     * Technically use JS bitwise operations at needed index.
     */
    value: function _eval() {
      var a = this.getInputPins()[0].getValue();

      // JS ~ operator applies bitwise Not for all bits:
      this.getOutputPins()[0].setValue(~a);
    }
  }]);

  return Not16;
}(BuiltInGate);

/**
 * Specification of the `Not16` gate.
 */


Not16.Spec = {
  name: 'Not16',

  description: 'Implements bitwise 16-bit Not ~ operation.',

  inputPins: [{ name: 'in', size: 16 }],

  outputPins: [{ name: 'out', size: 16 }],

  truthTable: TRUTH_TABLE
};

module.exports = Not16;