/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BuiltInGate = require('../BuiltInGate');

var _require = require('../../../util/numbers'),
    int16Table = _require.int16Table;

/**
 * Canonical truth table for the `Or8Way` gate.
 */


var TRUTH_TABLE = int16Table([{ in: 0, out: 0 }, { in: 255, out: 1 }, { in: 16, out: 1 }, { in: 1, out: 1 }, { in: 38, out: 1 }]);

/**
 * Or of 8 inputs into 1 output.
 * out=1 if one or more of the inputs is 1 and 0 otherwise.
 */

var Or8Way = function (_BuiltInGate) {
  _inherits(Or8Way, _BuiltInGate);

  function Or8Way() {
    _classCallCheck(this, Or8Way);

    return _possibleConstructorReturn(this, (Or8Way.__proto__ || Object.getPrototypeOf(Or8Way)).apply(this, arguments));
  }

  _createClass(Or8Way, [{
    key: 'eval',

    /**
     * IN in[8];
     * OUT out;
     *
     * out = (in[0] | in[1] | ... | in[7])
     *
     * Abstract:
     *
     *   Or(a=in[0], b=in[1], out=out1);
     *   Or(a=out1, b=in[2], out=out2);
     *   ...
     */
    value: function _eval() {
      var _in = this.getInputPins()[0].getValue();

      this.getOutputPins()[0].setValue(_in === 0 ? 0 : 1);
    }
  }]);

  return Or8Way;
}(BuiltInGate);

/**
 * Specification of the `Or8Way` gate.
 */


Or8Way.Spec = {
  name: 'Or8Way',

  description: 'Or | of 8 inputs into 1 output.',

  inputPins: [{ name: 'in', size: 8 }],

  outputPins: ['out'],

  truthTable: TRUTH_TABLE
};

module.exports = Or8Way;