/**
 * The MIT License (MIT)
 * Copyright (c) 2017-present Dmitry Soshnikov <dmitry.soshnikov@gmail.com>
 */

'use strict';

var BuiltInGate = require('./BuiltInGate');
var BuiltInGates = require('./builtin-gates');
var Clock = require('./Clock');
var CompositeGate = require('./CompositeGate');
var Gate = require('./Gate');
var HDLClassFactory = require('./HDLClassFactory');
var Pin = require('./Pin');
var ScriptInterpreter = require('./scripting/ScriptInterpreter');

/**
 * Simulates hardware chips (gates) evaluation.
 */
var HardwareEmulator = {
  /**
   * Expose `Clock` class.
   */
  Clock: Clock,

  /**
   * Expose `Gate` class.
   */
  Gate: Gate,

  /**
   * Expose `Pin` class.
   */
  Pin: Pin,

  /**
   * Expose `BuiltInGate` class.
   */
  BuiltInGate: BuiltInGate,

  /**
   * Expose `CompositeGate` class.
   */
  CompositeGate: CompositeGate,

  /**
   * Expose `HDLClassFactory` object.
   */
  HDLClassFactory: HDLClassFactory,

  /**
   * Expose a map of all built-in gates.
   */
  BuiltInGates: BuiltInGates,

  /**
   * Script interpreter module.
   */
  ScriptInterpreter: ScriptInterpreter
};

module.exports = HardwareEmulator;